@@ -677,6 +677,30 @@ check_votes(qd_ctx *ctx, node_info_t *ni, int max, disk_msg_t *msg)
 	return 0;
 }
 
+
+static int
+register_device(qd_ctx *ctx)
+{
+	return cman_register_quorum_device(
+			ctx->qc_cman_admin,
+			(ctx->qc_flags&RF_CMAN_LABEL) ?
+				ctx->qc_cman_label : ctx->qc_device,
+			(!(ctx->qc_flags & RF_MASTER_WINS) ||
+			 ctx->qc_status == S_MASTER) ?
+				ctx->qc_votes : 0);
+}
+
+
+static int 
+adjust_votes(qd_ctx *ctx)
+{
+	if (!(ctx->qc_flags & RF_MASTER_WINS))
+		return 0;
+
+	return register_device(ctx);
+}
+
+
 static void
 print_node_info(FILE *fp, node_info_t *ni)
 {
@@ -968,6 +992,7 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 				       "downgrading\n",
 				       score, score_max, score_req);
 				ctx->qc_status = S_NONE;
+				adjust_votes(ctx);
 				msg.m_msg = M_NONE;
 				++msg.m_seq;
 				bid_pending = 0;
@@ -1006,6 +1031,7 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 
 			/* Handle just like a recent upgrade */
 			ctx->qc_status = S_RUN;
+			adjust_votes(ctx);
 			upgrade = ctx->qc_upgrade_wait;
 			bid_pending = 0;
 			msg.m_msg = M_NONE;
@@ -1055,6 +1081,7 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 				logt_print(LOG_INFO,
 				       "Assuming master role\n");
 				ctx->qc_status = S_MASTER;
+				adjust_votes(ctx);
 			case 2:
 				msg.m_msg = M_NONE;
 			case 1:
@@ -1107,9 +1134,11 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 						"Halting qdisk operations\n");
 					return -1;
 				}
-				if (!errors && 
-				    (!(ctx->qc_flags & RF_MASTER_WINS)))
-					cman_poll_quorum_device(ctx->qc_cman_admin, 1);
+				if (!errors) {
+				    	cman_poll_quorum_device(
+						ctx->qc_cman_admin,
+						1);
+				}
 			}
 		}
 		
@@ -1338,17 +1367,99 @@ get_log_config_data(int ccsfd)
 }
 
 
+/*
+ * return number of nodes - 1 on success
+ *        -1 on generic error
+ *        -2 if one of the node votes is != 1
+ */
+static int
+auto_qdisk_votes(int desc)
+{
+	int ret = 1;
+	char buf[PATH_MAX];
+	char *name;
+
+	while (1) {
+		int votes=0;
+
+		snprintf(buf, sizeof(buf)-1,
+			"/cluster/clusternodes/clusternode[%d]/@votes", ret);
+
+		name = NULL;
+		if (ccs_get(desc, buf, &name) != 0)
+			break;
+
+		votes=atoi(name);
+		if (votes != 1) {
+			free(name);
+
+			snprintf(buf, sizeof(buf)-1,
+			    "/cluster/clusternodes/clusternode[%d]/@name",
+			    ret);
+
+			if (ccs_get(desc, buf, &name) == 0) {
+				logt_print(LOG_ERR, "%s's vote count is %d\n",
+					   name, votes);
+				free(name);
+			}
+
+			logt_print(LOG_ERR, "Set all node vote counts to 1 "
+				   "or specify qdiskd's votes\n");
+			return -2;
+		}
+
+		free(name);
+		ret++;
+ 	}
+ 
+	// adjust count (one from init and one from the node count)
+	ret = ret - 2;
+ 
+	if (ret <= 0)
+		logt_print(LOG_ERR, "Unable to determine qdiskd votes "
+			   "automatically\n");
+	else
+		logt_print(LOG_DEBUG, "Setting votes to %d\n", ret);
+ 
+ 	return (ret);
+}
+
+
 static int
 get_dynamic_config_data(qd_ctx *ctx, int ccsfd)
 {
 	char *val = NULL;
 	char query[256];
+	int old_votes = 0, found = 0;
 
 	if (ccsfd < 0)
 		return -1;
 
 	logt_print(LOG_DEBUG, "Loading dynamic configuration\n");
 
+	/* Check label / device presence.  If it disappeared, we need to exit */
+	if (ctx->qc_config) {
+		val = NULL;
+		snprintf(query, sizeof(query), "/cluster/quorumd/@device");
+		found = ccs_get(ccsfd, query, &val);
+		if (found != 0) {
+			val = NULL;
+			snprintf(query, sizeof(query), "/cluster/quorumd/@label");
+			found = ccs_get(ccsfd, query, &val);
+			free(val);
+		}
+
+		if (found != 0) {
+			logt_print(LOG_NOTICE,
+				   "Quorum device removed from the configuration."
+				   "  Shutting down.\n");
+			ctx->qc_votes = 0;
+			register_device(ctx);
+			_running = 0;
+			return -1;
+		}
+	}
+
 	/* Get status file */
 	snprintf(query, sizeof(query), "/cluster/quorumd/@status_file");
 	if (ccs_get(ccsfd, query, &val) == 0) {
@@ -1464,44 +1575,59 @@ get_dynamic_config_data(qd_ctx *ctx, int ccsfd)
 		free(val);
 	}
 
+	/* Get votes */
+	if (ctx->qc_config) {
+		old_votes = ctx->qc_votes;
+		ctx->qc_flags &= ~RF_AUTO_VOTES;
+	}
+
+	snprintf(query, sizeof(query), "/cluster/quorumd/@votes");
+	if (ccs_get(ccsfd, query, &val) == 0) {
+		ctx->qc_votes = atoi(val);
+		free(val);
+		if (ctx->qc_votes < 0)
+			ctx->qc_votes = 0;
+	} else {
+		ctx->qc_votes = auto_qdisk_votes(ccsfd);
+		if (ctx->qc_votes < 0) {
+			if (ctx->qc_config) {
+				logt_print(LOG_WARNING, "Unable to determine "
+					   "new vote value; retaining old "
+					   "value of %d\n", old_votes);
+				ctx->qc_votes = old_votes;
+			} else {
+				/* During startup, this is fatal */
+				return -1;
+			}
+		} else {
+			ctx->qc_flags |= RF_AUTO_VOTES;
+		}
+	}
+
+	if (ctx->qc_config && old_votes != ctx->qc_votes) {
+		logt_print(LOG_DEBUG, "Changing vote count from %d to %d\n",
+			   old_votes, ctx->qc_votes);
+
+		if (ctx->qc_flags & RF_AUTO_MASTER_WINS) {
+			logt_print(LOG_DEBUG, "Vote count changed! "
+				   "Disabling master-wins\n");
+			ctx->qc_flags &= ~(RF_MASTER_WINS|RF_AUTO_MASTER_WINS);
+		}
+
+		/*
+		 * Here, we are reconfiguring _only_ the votes.  The 
+		 * label / cman runflags do not change during reconfiguration
+		 *
+		 * This only works after we have already gotten static
+		 * configuration data during initial startup.
+		 */
+		register_device(ctx);
+	}
+
 	return 0;
 }
 
 
-static int
-auto_qdisk_votes(int desc)
-{
-	int x = 0, ret = 0;
-	char buf[128];
-	char *name;
-
-	if (desc < 0) {
-		return 1;
-	}
-
-	while (++x) {
-		snprintf(buf, sizeof(buf)-1,
-			"/cluster/clusternodes/clusternode[%d]/@name", x);
-
-		name = NULL;
-		if (ccs_get(desc, buf, &name) != 0)
-			break;
-
-		free(name);
-		ret = x;
-	}
-
-	--ret;
-	if (ret <= 0) {
-		ret = 1;
-	}
-
-	logt_print(LOG_DEBUG, "Setting votes to %d\n", ret);
-
-	return (ret);
-}
-
-
 static int
 get_static_config_data(qd_ctx *ctx, int ccsfd) 
 {
@@ -1531,6 +1657,7 @@ get_static_config_data(qd_ctx *ctx, int ccsfd)
 			logt_print(LOG_DEBUG, "Token timeout %d is too fast "
 				   "to use with qdiskd!\n",
 				   ctx->qc_token_timeout);
+			return -1;
 		} 
 	} else {
 		ctx->qc_token_timeout = DEFAULT_TOKEN_TIMEOUT;
@@ -1550,7 +1677,7 @@ get_static_config_data(qd_ctx *ctx, int ccsfd)
 	}
 
 	if (ctx->qc_tko < 4) {
-		logt_print(LOG_ERR, "Quorum disk TKO (%d) is too low!\n",
+		logt_print(LOG_WARNING, "Quorum disk TKO (%d) is too low!\n",
 			   ctx->qc_tko);
 	}
 
@@ -1586,30 +1713,26 @@ get_static_config_data(qd_ctx *ctx, int ccsfd)
 	if (ctx->qc_master_wait <= ctx->qc_tko_up)
 		ctx->qc_master_wait = ctx->qc_tko_up + 1;
 
+	logt_print(LOG_DEBUG, "Timings: %d tko, %d interval\n",
+		   ctx->qc_tko, ctx->qc_interval);
+	logt_print(LOG_DEBUG, "Timings: %d tko_up, %d master_wait, "
+		   "%d upgrade_wait\n",
+		   ctx->qc_tko_up, ctx->qc_master_wait, ctx->qc_upgrade_wait);
+
 	qdisk_fo = ctx->qc_interval * (ctx->qc_master_wait +
 				ctx->qc_upgrade_wait +
 				ctx->qc_tko) * 1000;
 	if (qdisk_fo >= ctx->qc_token_timeout) {
-		logt_print(LOG_WARNING, "Quorum disk timings are too slow for "
+		logt_print(LOG_ERR, "Quorum disk timings are too slow for "
 			   "configured token timeout\n");
-		logt_print(LOG_WARNING, " * Totem Token timeout: %dms\n",
+		logt_print(LOG_ERR, " * Totem Token timeout: %dms\n",
 			   ctx->qc_token_timeout);
-		logt_print(LOG_WARNING, " * Min. Master recovery time: %dms\n",
+		logt_print(LOG_ERR, " * Min. Master recovery time: %dms\n",
 			   qdisk_fo);
-		logt_print(LOG_WARNING,
+		logt_print(LOG_ERR,
 			   "Please set token timeout to at least %dms\n",
 			   qdisk_fo + (ctx->qc_interval * 1000));
-	}
-
-	/* Get votes */
-	snprintf(query, sizeof(query), "/cluster/quorumd/@votes");
-	if (ccs_get(ccsfd, query, &val) == 0) {
-		ctx->qc_votes = atoi(val);
-		free(val);
-		if (ctx->qc_votes < 0)
-			ctx->qc_votes = 0;
-	} else {
-		ctx->qc_votes = auto_qdisk_votes(ccsfd);
+		return -1;
 	}
 
 	/* Get device */
@@ -1624,6 +1747,12 @@ get_static_config_data(qd_ctx *ctx, int ccsfd)
 		ctx->qc_label = val;
 	}
 
+	if (!ctx->qc_device && !ctx->qc_label) {
+		logt_print(LOG_ERR, "No device or label specified; cannot "
+			   "run QDisk services.\n");
+		return -1;
+	}
+
 	/* Get min score */
 	snprintf(query, sizeof(query), "/cluster/quorumd/@min_score");
 	if (ccs_get(ccsfd, query, &val) == 0) {
@@ -1665,7 +1794,6 @@ get_static_config_data(qd_ctx *ctx, int ccsfd)
 		free(val);
 	}
 
-
 	return 0;
 }
 
@@ -1676,7 +1804,7 @@ get_static_config_data(qd_ctx *ctx, int ccsfd)
 static int
 get_config_data(qd_ctx *ctx, struct h_data *h, int maxh, int *cfh)
 {
-	int ccsfd = -1;
+	int ccsfd = -1, ret = -1;
 
 	ccsfd = ccs_connect();
 	if (ccsfd < 0) {
@@ -1700,14 +1828,20 @@ get_config_data(qd_ctx *ctx, struct h_data *h, int maxh, int *cfh)
 		ctx->qc_max_error_cycles = 0;
 	}
 	
-	if (ctx->qc_config ||
-	    get_dynamic_config_data(ctx, ccsfd) < 0)
+	if (get_dynamic_config_data(ctx, ccsfd) < 0) {
 		goto out;
+	}
+
+	if (ctx->qc_config) {
+		ret = 0;
+		goto out;
+	}
 
 	ctx->qc_config = 1;
 
-	if (get_static_config_data(ctx, ccsfd) < 0)
+	if (get_static_config_data(ctx, ccsfd) < 0) {
 		goto out;
+	}
 
 	*cfh = configure_heuristics(ccsfd, h, maxh);
 
@@ -1716,20 +1850,31 @@ get_config_data(qd_ctx *ctx, struct h_data *h, int maxh, int *cfh)
 			logt_print(LOG_WARNING, "Master-wins mode disabled\n");
 			ctx->qc_flags &= ~RF_MASTER_WINS;
 		}
+	} else {
+		if (ctx->qc_flags & RF_AUTO_VOTES &&
+		    !(ctx->qc_flags & RF_MASTER_WINS) &&
+		    ctx->qc_votes == 1) { 
+			/* Two node cluster, no heuristics, 1 vote for
+			 * quorum disk daemon.  Safe to enable master-wins.
+			 * In fact, qdiskd without master-wins in this config
+			 * is a waste of resources.
+			 */
+			ctx->qc_flags |= RF_MASTER_WINS | RF_AUTO_MASTER_WINS;
+			logt_print(LOG_INFO, "Enabling master-wins mode for "
+				   "simple two-node cluster\n");
+		}
 	}
 
+	ret = 0;
+
 	logt_print(LOG_DEBUG, "Quorum Daemon: %d heuristics, "
 		   "%d interval, %d tko, %d votes\n",
 		   *cfh, ctx->qc_interval, ctx->qc_tko, ctx->qc_votes);
-	logt_print(LOG_DEBUG, "%d tko_up, %d master_wait, "
-		   "%d upgrade_wait\n",
-		   ctx->qc_tko_up, ctx->qc_master_wait, ctx->qc_upgrade_wait);
-out:
 	logt_print(LOG_DEBUG, "Run Flags: %08x\n", ctx->qc_flags);
-
+out:
 	ccs_disconnect(ccsfd);
 
-	return 0;
+	return ret;
 }
 
 
@@ -1925,28 +2070,14 @@ main(int argc, char **argv)
 	if (!_running)
 		goto out;
 	
-	cman_register_quorum_device(ctx.qc_cman_admin,
-				    (ctx.qc_flags&RF_CMAN_LABEL)? 
-				        ctx.qc_cman_label:
-                                        ctx.qc_device,
-				    ctx.qc_votes);
-	/*
-		XXX this always returns -1 / EBUSY even when it works?!!!
-		
-	if ((rv = cman_register_quorum_device(ctx.qc_cman_admin, ctx.qc_device,
-					      ctx.qc_votes)) < 0) {
-		logt_print(LOG_CRIT,
-				 "Could not register %s with CMAN; "
-				 "return = %d; error = %s\n",
-				 ctx.qc_device, rv, strerror(errno));
-		goto out;
-	}
-	*/
+	/* This registers the quorum device */
+	register_device(&ctx);
 
 	io_nanny_start(ctx.qc_tko * ctx.qc_interval);
 
 	if (quorum_loop(&ctx, ni, MAX_NODES_DISK) == 0) {
 		/* Only clean up if we're exiting w/o error) */
+		logt_print(LOG_NOTICE, "Unregistering quorum device.\n");
 		cman_unregister_quorum_device(ctx.qc_cman_admin);
 		quorum_logout(&ctx);
 	}
