@@ -133,7 +133,7 @@ static void warm_fuzzy_stuff(uint64_t wfsblock, int force, int save)
 		if (last_fs_block) {
 			printf("\r");
 			if (save) {
-				percent = (block * 100) / last_fs_block;
+				percent = (wfsblock * 100) / last_fs_block;
 				printf("%" PRIu64 " metadata blocks (%"
 				       PRIu64 "%%) processed, ", wfsblock,
 				       percent);
@@ -299,6 +299,7 @@ static void save_indirect_blocks(int out_fd, osi_list_t *cur_list,
 		if (height != hgt) { /* If not at max height */
 			nbh = bread(&sbd, indir_block);
 			osi_list_add_prev(&nbh->b_altlist, cur_list);
+			brelse(nbh);
 		}
 	} /* for all data on the indirect block */
 }
@@ -586,12 +587,14 @@ void savemeta(char *out_fn, int saveoption)
 		brelse(lbh);
 	}
 	if (!slow) {
+		int sane;
+
 		printf("Reading resource groups...");
 		fflush(stdout);
 		if (gfs1)
 			slow = gfs1_ri_update(&sbd, 0, &rgcount, 0);
 		else
-			slow = ri_update(&sbd, 0, &rgcount);
+			slow = ri_update(&sbd, 0, &rgcount, &sane);
 		printf("Done.\n\n");
 		fflush(stdout);
 	}
@@ -906,5 +909,5 @@ void restoremeta(const char *in_fn, const char *out_device,
 	if (!printblocksonly)
 		close(sbd.device_fd);
 
-	exit(0);
+	exit(error);
 }
