@@ -27,7 +27,7 @@ struct dup_handler {
 };
 
 static int check_metalist(struct gfs2_inode *ip, uint64_t block,
-			  struct gfs2_buffer_head **bh, void *private);
+			  struct gfs2_buffer_head **bh, int h, void *private);
 static int check_data(struct gfs2_inode *ip, uint64_t block, void *private);
 static int check_eattr_indir(struct gfs2_inode *ip, uint64_t block,
 			     uint64_t parent, struct gfs2_buffer_head **bh,
@@ -74,7 +74,7 @@ struct metawalk_fxns find_dirents = {
 };
 
 static int check_metalist(struct gfs2_inode *ip, uint64_t block,
-			  struct gfs2_buffer_head **bh, void *private)
+			  struct gfs2_buffer_head **bh, int h, void *private)
 {
 	return add_duplicate_ref(ip, block, ref_as_meta, 1, INODE_VALID);
 }
@@ -168,12 +168,13 @@ static int find_dentry(struct gfs2_inode *ip, struct gfs2_dirent *de,
 		       struct gfs2_buffer_head *bh, char *filename,
 		       uint16_t *count, void *priv)
 {
-	struct osi_node *n;
+	struct osi_node *n, *next = NULL;
 	osi_list_t *tmp2;
 	struct duptree *b;
 	int found;
 
-	for (n = osi_first(&dup_blocks); n; n = osi_next(n)) {
+	for (n = osi_first(&dup_blocks); n; n = next) {
+		next = osi_next(n);
 		b = (struct duptree *)n;
 		found = 0;
 		osi_list_foreach(tmp2, &b->ref_invinode_list) {
@@ -196,7 +197,8 @@ static int find_dentry(struct gfs2_inode *ip, struct gfs2_dirent *de,
 }
 
 static int clear_dup_metalist(struct gfs2_inode *ip, uint64_t block,
-			      struct gfs2_buffer_head **bh, void *private)
+			      struct gfs2_buffer_head **bh, int h,
+			      void *private)
 {
 	struct dup_handler *dh = (struct dup_handler *) private;
 	struct duptree *d;
@@ -245,21 +247,21 @@ static int clear_dup_metalist(struct gfs2_inode *ip, uint64_t block,
 
 static int clear_dup_data(struct gfs2_inode *ip, uint64_t block, void *private)
 {
-	return clear_dup_metalist(ip, block, NULL, private);
+	return clear_dup_metalist(ip, block, NULL, 0, private);
 }
 
 static int clear_dup_eattr_indir(struct gfs2_inode *ip, uint64_t block,
 				 uint64_t parent, struct gfs2_buffer_head **bh,
 				 void *private)
 {
-	return clear_dup_metalist(ip, block, NULL, private);
+	return clear_dup_metalist(ip, block, NULL, 0, private);
 }
 
 static int clear_dup_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 				uint64_t parent, struct gfs2_buffer_head **bh,
 				void *private)
 {
-	return clear_dup_metalist(ip, block, NULL, private);
+	return clear_dup_metalist(ip, block, NULL, 0, private);
 }
 
 static int clear_eattr_entry (struct gfs2_inode *ip,
@@ -312,7 +314,7 @@ static int clear_eattr_extentry(struct gfs2_inode *ip, uint64_t *ea_data_ptr,
 {
 	uint64_t block = be64_to_cpu(*ea_data_ptr);
 
-	return clear_dup_metalist(ip, block, NULL, private);
+	return clear_dup_metalist(ip, block, NULL, 0, private);
 }
 
 /* Finds all references to duplicate blocks in the metadata */
@@ -600,7 +602,7 @@ int pass1b(struct gfs2_sbd *sbp)
 	struct duptree *b;
 	uint64_t i;
 	uint8_t q;
-	struct osi_node *n;
+	struct osi_node *n, *next = NULL;
 	int rc = FSCK_OK;
 
 	log_info( _("Looking for duplicate blocks...\n"));
@@ -650,7 +652,8 @@ int pass1b(struct gfs2_sbd *sbp)
 	 * it later */
 	log_info( _("Handling duplicate blocks\n"));
 out:
-        for (n = osi_first(&dup_blocks); n; n = osi_next(n)) {
+        for (n = osi_first(&dup_blocks); n; n = next) {
+		next = osi_next(n);
                 b = (struct duptree *)n;
 		if (!skip_this_pass && !rc) /* no error & not asked to skip the rest */
 			handle_dup_blk(sbp, b);
